@@ -1,93 +1,15 @@
-#include <R.h>
-#include <Rinternals.h>
-#include <R_ext/Error.h>
-#include <R_ext/Memory.h>
+#include "Rcommon.h"
 #include "eaf.h"
 
-#ifndef DEBUG
-#define DEBUG 0
-#endif
-
-#if DEBUG >= 1
-#define DEBUG1(X) X;(void)0
-#else  
-#define DEBUG1(X) (void)0
-#endif
-
-#define CHECK_ARG_IS_INT_VECTOR(A)					\
-    if (!isInteger(A) || !isVector(A))					\
-	error("Argument '" #A "' is not an integer vector");
-
-/* The C API of R is awfully ugly and unpractical (and poorly
-   documented). These wrappers make it a little more bearable. */
-
-#define Rexp(VAR) Rexp_##VAR
-
-#define new_real_matrix(DOUBLEVAR, DIM1, DIM2)                                 \
-    SEXP Rexp_##DOUBLEVAR; double *DOUBLEVAR;                                  \
-    PROTECT(Rexp_##DOUBLEVAR = allocMatrix(REALSXP, (DIM1), (DIM2)));          \
-    nprotected++; DOUBLEVAR = REAL(Rexp_##DOUBLEVAR)
-
-#define new_real_vector(DOUBLEVAR, DIM)                                        \
-    SEXP Rexp_##DOUBLEVAR; double *DOUBLEVAR;                                  \
-    PROTECT(Rexp_##DOUBLEVAR = allocVector(REALSXP, (DIM)));                   \
-    nprotected++; DOUBLEVAR = REAL(Rexp_##DOUBLEVAR)
-
-#define new_string_vector(VAR, DIM)                                            \
-    SEXP Rexp_##VAR; int Rexp_##VAR##_len = 0;                                 \
-    PROTECT(Rexp_##VAR = allocVector(STRSXP, (DIM)));                          \
-    nprotected++
-
-#define string_vector_push_back(VAR, ELEMENT)                                  \
-    SET_STRING_ELT(Rexp_##VAR, Rexp_##VAR##_len, mkChar(ELEMENT));             \
-    Rexp_##VAR##_len++
-
-#define new_list(LISTVAR, LENGTH)                                              \
-    SEXP Rexp_##LISTVAR; int Rexp_##LISTVAR##_len = 0;                         \
-    PROTECT(Rexp_##LISTVAR = allocVector(VECSXP, (LENGTH)));                   \
-    ++nprotected
-
-#define list_len(VAR) Rexp_##VAR##_len
-
-#define list_push_back(LISTVAR, ELEMENT)                                       \
-    SET_VECTOR_ELT(Rexp_##LISTVAR, Rexp_##LISTVAR##_len, Rexp_##ELEMENT);      \
-    Rexp_##LISTVAR##_len++
-
-#define set_names(VAR, NAMES)                                                  \
-    setAttrib(Rexp_##VAR, R_NamesSymbol, Rexp_##NAMES)
-
-#define set_attribute(VAR, ATTRIBUTE, VALUE)                                   \
-    setAttrib(Rexp_##VAR, install(ATTRIBUTE), Rexp_##VALUE)
-
-/*
- * Unpack an integer vector stored in SEXP S.
- */
-#define SEXP_2_INT_VECTOR(S, I, N)               \
-    CHECK_ARG_IS_INT_VECTOR(S);                  \
-    int *I = INTEGER(S);                         \
-    const R_len_t N = length(S);
-
-#define SEXP_2_INT(S,var)                                               \
-    int var = asInteger(S);                                             \
-    if (var == NA_INTEGER)                                              \
-        error ("Argument '" #S "' is not an integer");
-
-#define SEXP_2_STRING(S,var)                                            \
-    if (!isString(S) || length(S) != 1)                                 \
-        error ("Argument '" #S "' is not a string");                    \
-    const char * var = CHAR(STRING_ELT(S,0));
-
-
 static eaf_t **
 compute_eaf_helper (SEXP DATA, int nobj, SEXP CUMSIZES, int nruns, 
-                    const int *percentile, int nlevels)
+                    const double *percentile, int nlevels)
 {
-    eaf_t **eaf;
     int k;
     SEXP_2_INT_VECTOR(CUMSIZES, cumsizes, cumsizes_len);
     if (cumsizes_len < nruns)
-        error("length of cumsizes (%d) is less than nruns (%d)",
-              cumsizes_len, nruns);
+        Rf_error("length of cumsizes (%d) is less than nruns (%d)",
+                 cumsizes_len, nruns);
 
     int *level;
 
@@ -104,9 +26,12 @@ compute_eaf_helper (SEXP DATA, int nobj, SEXP CUMSIZES, int nruns,
 
     double *data = REAL(DATA);
 
-    DEBUG1(
-        Rprintf ("attsurf ({(%f,%f)...}, %d, { %d",
-                 data[0], data[1], nobj, cumsizes[0]);
+    DEBUG2(
+        Rprintf ("attsurf ({(%f, %f", data[0], data[1]);
+        for (k = 2; k < nobj; k++) {
+            Rprintf (", %f", data[k]);
+        }
+        Rprintf (")...}, %d, { %d", nobj, cumsizes[0]);
         for (k = 1; k < nruns; k++) {
             Rprintf (", %d", cumsizes[k]);
         }
@@ -117,10 +42,10 @@ compute_eaf_helper (SEXP DATA, int nobj, SEXP CUMSIZES, int nruns,
         Rprintf ("}, %d)\n", nlevels);
         );
 
-    eaf = attsurf (data, nobj, cumsizes, nruns, level, nlevels);
+    eaf_t **eaf = attsurf (data, nobj, cumsizes, nruns, level, nlevels);
     free (level);
 
-    DEBUG1(
+    DEBUG2(
         Rprintf ("eaf computed\n");
         for (k = 0; k < nlevels; k++) {
             Rprintf ("eaf[%d] = %d\n", k, eaf[k]->size);
@@ -129,45 +54,37 @@ compute_eaf_helper (SEXP DATA, int nobj, SEXP CUMSIZES, int nruns,
     return eaf;
 }
 
-SEXP compute_eaf_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
-                   SEXP PERCENTILES);
-
 SEXP
-compute_eaf_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
-              SEXP PERCENTILE)
+compute_eaf_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS, SEXP PERCENTILE)
 {
-    eaf_t **eaf;
-    int k;
     SEXP_2_INT(NOBJ, nobj);
     SEXP_2_INT(NRUNS, nruns);
-    SEXP_2_INT_VECTOR(PERCENTILE, percentile, nlevels);
+    SEXP_2_DOUBLE_VECTOR(PERCENTILE, percentile, nlevels);
 
-    eaf = compute_eaf_helper(DATA, nobj, CUMSIZES, nruns, percentile, nlevels);
-
-    int totalpoints = 0;
-    for (k = 0; k < nlevels; k++) {
-        totalpoints += eaf[k]->size;
-    }
+    eaf_t **eaf = compute_eaf_helper(DATA, nobj, CUMSIZES, nruns, percentile, nlevels);
+    int totalpoints = eaf_totalpoints (eaf, nlevels);
 
     SEXP mat;
-    double *rmat;
-    PROTECT(mat = allocMatrix(REALSXP, totalpoints, nobj + 1));
-    rmat = REAL(mat);
+    PROTECT(mat = Rf_allocMatrix(REALSXP, totalpoints, nobj + 1));
+    double * rmat = REAL(mat);
 
     int pos = 0;
+    int k;
     for (k = 0; k < nlevels; k++) {
         int npoints = eaf[k]->size;
-        int i;
 
-        DEBUG1(
+        DEBUG2(
             int totalsize = npoints * nobj;
             Rprintf ("totalpoints eaf[%d] = %d\n", k, totalsize)
             );
 
+        int i;
         for (i = 0; i < npoints; i++) {
-            rmat[pos] = eaf[k]->data[i * nobj];
-            rmat[pos + totalpoints] = eaf[k]->data[1 + i * nobj];
-            rmat[pos + 2 * totalpoints] = percentile[k];
+            int j;
+            for (j = 0; j < nobj; j++) {
+                rmat[pos + j * totalpoints] = eaf[k]->data[j + i * nobj];
+            }
+            rmat[pos + nobj * totalpoints] = percentile[k];
             pos++;
         }
         eaf_delete (eaf[k]);
@@ -177,60 +94,54 @@ compute_eaf_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
     return mat;
 }
 
-
-SEXP compute_eafdiff_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
-                       SEXP INTERVALS);
-
 SEXP 
 compute_eafdiff_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
                   SEXP INTERVALS)
 {
-    eaf_t **eaf;
-    int k;
     SEXP_2_INT(NOBJ, nobj);
     SEXP_2_INT(NRUNS, nruns);
     SEXP_2_INT(INTERVALS, intervals);
-
-    eaf = compute_eaf_helper(DATA, nobj, CUMSIZES, nruns, NULL, nruns);
+    int k;
+    
+    eaf_t **eaf = compute_eaf_helper(DATA, nobj, CUMSIZES, nruns, NULL, nruns);
 
     int nsets1 = nruns / 2;
     int nsets2 = nruns - nsets1;
+    int totalpoints = eaf_totalpoints (eaf, nruns);
 
-    int totalpoints = 0;
-    for (k = 0; k < nruns; k++) {
-        totalpoints += eaf[k]->size;
-    }
     SEXP mat;
-    double *rmat;
-    PROTECT(mat = allocMatrix(REALSXP, totalpoints, nobj + 1));
-    rmat = REAL(mat);
+    PROTECT(mat = Rf_allocMatrix(REALSXP, totalpoints, nobj + 1));
+    double *rmat = REAL(mat);
 
     int pos = 0;
     for (k = 0; k < nruns; k++) {
         int npoints = eaf[k]->size;
-        int i;
+        int i, j;
 
-        DEBUG1(
+        DEBUG2(
             int totalsize = npoints * nobj;
             Rprintf ("totalpoints eaf[%d] = %d\n", k, totalsize)
             );
 
+        // FIXME: Find the most efficient order of the loop.
         for (i = 0; i < npoints; i++) {
-            rmat[pos] = eaf[k]->data[i * nobj];
-            rmat[pos + totalpoints] = eaf[k]->data[1 + i * nobj];
+            for (j = 0; j < nobj; j++) {
+                rmat[pos + j * totalpoints] = eaf[k]->data[j + i * nobj];
+            }
             pos++;
         }
     }
-
-    pos += totalpoints;
+    pos += (nobj - 1) * totalpoints;
     for (k = 0; k < nruns; k++) {
         int i;
         int npoints = eaf[k]->size;
         for (i = 0; i < npoints; i++) {
             int count_left;
             int count_right;
-            attained_left_right (eaf[k]->attained + i * eaf[k]->nruns,
-                                 nruns/2, nruns, &count_left, &count_right);
+            /* bit_array_check(bit_array_offset(eaf[k]->bit_attained,i, eaf[k]->nruns), */
+            /*                 eaf[k]->attained + i * eaf[k]->nruns, npoints, nruns); */
+            attained_left_right (bit_array_offset(eaf[k]->bit_attained, i, eaf[k]->nruns),
+                                 nsets1, nruns, &count_left, &count_right);
             rmat[pos] = intervals * (double) ((count_left / (double) nsets1) - 
                                               (count_right / (double) nsets2));
             pos++;
@@ -250,6 +161,7 @@ static int polygon_len  (const double *src, int nobj)
     src += nobj;
     return (src - src_orig) / nobj;
 }
+
 static int polygon_copy (double *dest, int start, int nrows, const double *src)
 {
     int len = start;
@@ -265,6 +177,52 @@ static int polygon_copy (double *dest, int start, int nrows, const double *src)
     return len - start;
 }
 
+SEXP compute_eafdiff_rectangles_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
+                                  SEXP INTERVALS);
+SEXP
+compute_eafdiff_rectangles_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
+                             SEXP INTERVALS)
+{
+    SEXP_2_INT(NOBJ, nobj);
+    SEXP_2_INT(NRUNS, nruns);
+    SEXP_2_INT(INTERVALS, intervals);
+    int nprotected = 0;
+    int k;
+    
+    eaf_t **eaf = compute_eaf_helper(DATA, nobj, CUMSIZES, nruns, NULL, nruns);
+    eaf_polygon_t * rects = eaf_compute_rectangles(eaf, nruns);
+    for (k = 0; k < nruns; k++)
+        eaf_delete (eaf[k]);
+    free(eaf);
+
+    const int division = nruns / 2;
+    int nrow = vector_int_size(&rects->col);
+    // Two points per row + color
+    new_real_matrix (result, nrow, 2 * nobj + 1);
+    double * p_xy = vector_objective_begin(&rects->xy);
+    for (k = 0; k < nrow; ++k) {
+        for (int i = 0; i < 2 * nobj; i++)
+            result[k + nrow * i] = (double) *(p_xy++);
+    }
+    vector_objective_dtor (&rects->xy);
+
+    for (k = 0; k < nrow; ++k) {
+        double color = vector_int_at(&rects->col, k);
+        // Each color is within [0, nruns / 2] or [-nruns / 2, 0]
+        result[k + nrow * 2 * nobj] = intervals * color / (double) division;
+    }
+    // FIXME: This may return duplicated rows, remove them.
+    vector_int_dtor (&rects->col);
+    free(rects);
+
+    const char* const colnames[] = {"xmin", "ymin", "xmax", "ymax", "diff"};
+    set_colnames(Rexp(result), colnames, 5);
+      
+    UNPROTECT (nprotected);
+    return Rexp(result);
+}
+
+
 SEXP compute_eafdiff_area_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
                             SEXP INTERVALS);
 SEXP 
@@ -273,15 +231,14 @@ compute_eafdiff_area_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
 {
     int nprotected = 0;
 
-    eaf_t **eaf;
     int k;
     SEXP_2_INT(NOBJ, nobj);
     SEXP_2_INT(NRUNS, nruns);
     SEXP_2_INT(INTERVALS, intervals);
 
-    eaf = compute_eaf_helper(DATA, nobj, CUMSIZES, nruns, NULL, nruns);
+    eaf_t **eaf = compute_eaf_helper(DATA, nobj, CUMSIZES, nruns, NULL, nruns);
 
-    eaf_polygon_t *p = eaf_compute_area (eaf, nruns);
+    eaf_polygon_t *p = eaf_compute_area(eaf, nruns);
 
     for (k = 0; k < nruns; k++)
         eaf_delete (eaf[k]);
@@ -291,7 +248,7 @@ compute_eafdiff_area_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
 
     int ncol = vector_int_size(&p->col);
 
-    DEBUG1(Rprintf ("ncol: %d\n", ncol));
+    DEBUG2(Rprintf ("ncol: %d\n", ncol));
 
     int left_ncol = 0, right_ncol = 0;
     int left_len = 0, right_len = 0;
@@ -300,11 +257,12 @@ compute_eafdiff_area_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
        on each side. */
     double * p_xy = vector_objective_begin(&p->xy);
     for (k = 0; k < ncol; k++) {
-        double color = vector_int_at(&p->col, k);
+        // Truncate colors to interval
+        int color = vector_int_at(&p->col, k) * intervals / (double) division;
         int len = polygon_len (p_xy, nobj);
         p_xy += len * nobj;
-        color = intervals * color / (double) division;
-        DEBUG1(Rprintf ("color: %d, len = %d\n", color, len));
+        DEBUG2(Rprintf ("color: %d, len = %d\n", color, len));
+        // First interval (-1, 1) is white
         if (color >= 1) {
             left_len += len;
             left_ncol++;
@@ -315,7 +273,7 @@ compute_eafdiff_area_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
         vector_int_set(&p->col, k, color);
     }
 
-    DEBUG1(Rprintf ("left_len: %d, right_len: %d, left_ncol: %d, right_ncol: %d\n", 
+    DEBUG2(Rprintf ("left_len: %d, right_len: %d, left_ncol: %d, right_ncol: %d\n", 
                     left_len, right_len, left_ncol, right_ncol));
 
     /* Now assign points to each side. */
@@ -341,7 +299,7 @@ compute_eafdiff_area_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
         } else if (color <= 1) {
             len = polygon_copy (right, right_len, right_npoints, p_xy);
             right_len += len;
-            right_col[right_ncol++] = (-color) + 1;
+            right_col[right_ncol++] = 1 - color;
         } else {
             len = polygon_len (p_xy, nobj);
         }
@@ -368,45 +326,31 @@ compute_eafdiff_area_C(SEXP DATA, SEXP NOBJ, SEXP CUMSIZES, SEXP NRUNS,
     return Rexp(poly);
 }
 
-SEXP read_data_sets (SEXP FILENAME);
-
 SEXP
 read_data_sets (SEXP FILENAME)
 {
     SEXP_2_STRING(FILENAME, filename);
-    objective_t *data = NULL;
-    int* cumsizes = NULL;
-    int nobj = 0, nruns = 0;
-
     /* Rprintf ("filename: %s\n", filename); */
-
+    objective_t * data = NULL;
+    int * cumsizes = NULL;
+    int nobj = 0, nruns = 0;
     read_objective_t_data (filename, &data, &nobj, &cumsizes, &nruns);
 
     const int ntotal = cumsizes[nruns - 1];
-    int * runtab = malloc (ntotal * sizeof(int));
-    int k, j, i;
-    for (k = 0, j = 0; k < ntotal; k++) {
-        if (k == cumsizes[j])
-            j++;
-        runtab[k] = j + 1;
-    }
 
     SEXP DATA;
-    PROTECT(DATA = allocMatrix(REALSXP, cumsizes[nruns-1], nobj + 1));
+    PROTECT(DATA = Rf_allocMatrix(REALSXP, cumsizes[nruns-1], nobj + 1));
     double *rdata = REAL(DATA);
-    int pos = 0;
-    for (j = 0; j < nobj; j++) {
-        for (i = 0; i < ntotal; i++) {
-            rdata[pos] = data[j + i * nobj];
-            pos++;
-        }
-    }
-    for (j = 0; j < ntotal; j++, pos++) {
-        rdata[pos] = runtab[j];
+    double_transpose (rdata, data, ntotal, nobj);
+
+    int k, j;
+    size_t pos = ntotal * nobj;
+    for (k = 0, j = 0; k < ntotal; k++, pos++) {
+        if (k == cumsizes[j]) j++;
+        rdata[pos] = j + 1;
     }
     free(data);
     free(cumsizes);
-    free(runtab);
     UNPROTECT (1);
     return DATA;
 }
